<?php

declare(strict_types=1);

namespace Dedi\SyliusDpdPickUpPlugin\Controller;

use DateTimeImmutable;
use Dedi\SyliusDpdPickUpPlugin\Form\Type\SearchPickUpType;
use Dedi\SyliusDpdPickUpPlugin\Locator\PickUpLocator;
use Dedi\SyliusDpdPickUpPlugin\Model\Location;
use Sylius\Component\Core\Model\AddressInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Order\Context\CartContextInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

class ChoosePickUpController extends AbstractController
{
    private readonly CartContextInterface $cartContext;
    private readonly PickUpLocator $locator;

    public function __construct(
        CartContextInterface $cartContext,
        PickUpLocator $locator,
    ) {
        $this->cartContext = $cartContext;
        $this->locator = $locator;
    }

    public function __invoke(Request $request): Response
    {
        /** @var OrderInterface $order */
        $order = $this->cartContext->getCart();

        /** @var AddressInterface $address */
        $address = $order->getShippingAddress();
        $location = Location::fromAddress($address);

        $form = $this->createForm(SearchPickUpType::class, $location);
        $form->handleRequest($request);

        if ($form->isSubmitted()) {
            if (!$form->isValid()) {
                $location = Location::fromAddress($address);
            }
        }

        $points = $this->locator->__invoke(
            $order->getId(),
            new DateTimeImmutable(),
            $location,
        );

        return $this->render('@DediSyliusDpdPickUpPlugin/Shop/Checkout/_choose_pickup.html.twig', [
            'order' => $order,
            'points' => $points,
            'form' => $form->createView(),
        ]);
    }
}
