<?php

declare(strict_types=1);

namespace Dedi\SyliusDpdPickUpPlugin\Service;

use Dedi\SyliusDpdPickUpPlugin\Enum\CorseProvinceCode;
use Dedi\SyliusDpdPickUpPlugin\Model\Pudo;
use Dedi\SyliusDpdPickUpPlugin\Specification\DoesPudoHasCorseDuSudPostalCodeSpecification;
use Dedi\SyliusDpdPickUpPlugin\Specification\DoesPudoHasHauteCorsePostalCodeSpecification;

/**
 * Guesses the province code thanks to the zipcode.
 * For instance for the zipcode 69009, it will returns FR-69
 */
class DefaultPudoToProvinceCodeConverter implements PudoToProvinceCodeConverterInterface
{
    private readonly DoesPudoHasCorseDuSudPostalCodeSpecification $pudoHasCorseDuSudPostalCodeSpecification;
    private readonly DoesPudoHasHauteCorsePostalCodeSpecification $pudoHasHauteCorsePostalCodeSpecification;

    public function __construct(
        DoesPudoHasCorseDuSudPostalCodeSpecification $pudoHasCorseDuSudPostalCodeSpecification,
        DoesPudoHasHauteCorsePostalCodeSpecification $pudoHasHauteCorsePostalCodeSpecification,
    ) {
        $this->pudoHasCorseDuSudPostalCodeSpecification = $pudoHasCorseDuSudPostalCodeSpecification;
        $this->pudoHasHauteCorsePostalCodeSpecification = $pudoHasHauteCorsePostalCodeSpecification;
    }

    public function transform(Pudo $pudo): string
    {
        if ($this->pudoHasCorseDuSudPostalCodeSpecification->__invoke($pudo)) {
            return CorseProvinceCode::CORSE_DU_SUD;
        }

        if ($this->pudoHasHauteCorsePostalCodeSpecification->__invoke($pudo)) {
            return CorseProvinceCode::HAUTE_CORSE;
        }

        return sprintf(
            'FR-%s',
            substr($pudo->zipCode, 0, 2),
        );
    }
}
